<?php

namespace Drupal\activitypub\Plugin\activitypub\type;

use ActivityPhp\Type;
use Drupal\activitypub\Entity\ActivityPubActivityInterface;
use Drupal\activitypub\Entity\Storage\ActivityPubActivityStorageInterface;
use Drupal\activitypub\Services\Type\TypePluginBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\Url;

/**
 * The ActivityPub static types.
 *
 * @ActivityPubType(
 *   id = "activitypub_static_types",
 *   label = @Translation("ActivityPub static types")
 * )
 */
class StaticTypes extends TypePluginBase {

  /**
   * {@inheritdoc}
   */
  public function isExposed() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function onActivitySave(ActivityPubActivityInterface $activity, $update = TRUE) {

    // Delete request for the same actor and object, just remove it.
    if ($activity->getType() == 'Delete' && $activity->getCollection() == ActivityPubActivityInterface::INBOX && $activity->getActor() == $activity->getObject()) {
      $activity->delete();
    }

    // Follow request.
    if ($activity->getType() == 'Follow' && !$update) {

      // Check if the actor already exists, if so, delete the incoming activity.
      /** @var \Drupal\activitypub\Entity\Storage\ActivityPubActivityStorageInterface $storage */
      $storage = $this->entityTypeManager->getStorage('activitypub_activity');
      $activities = $storage->getActivities(['type' => 'Follow', 'actor' => $activity->getActor(), 'object' => $activity->getObject(), '!id' => $activity->id()]);
      if (!empty($activities)) {
        $deleteFollow = array_shift($activities);
        $deleteFollow->delete();
      }
      else {
        $values = [
          'uid' => $activity->getOwnerId(),
          'collection' => ActivityPubActivityInterface::OUTBOX,
          'config_id' => 'accept',
          'type' => 'Accept',
          'actor' => $activity->getObject(),
          'object' => $activity->getActor(),
          'external_id' => $activity->getExternalId(),
          'status' => 0,
        ];

        /** @var \Drupal\activitypub\Entity\ActivityPubActivityInterface $outboxActivity */
        $outboxActivity = $this->entityTypeManager->getStorage('activitypub_activity')->create($values);
        $outboxActivity->save();
        $this->activityPubProcessClient->createQueueItem($outboxActivity);
      }
    }

    // Undo request.
    if ($activity->getType() == 'Undo' && !$update) {

      $payload = @json_decode($activity->getPayLoad(), TRUE);
      if (isset($payload['object']) && isset($payload['object']['type']) && $payload['object']['type'] == 'Follow') {
        $activities = $this->entityTypeManager->getStorage('activitypub_activity')->loadByProperties(['type' => 'Follow', 'actor' => $activity->getActor(), 'object' => $activity->getObject()]);
        if ($activities) {
          $deleteFollow = array_shift($activities);
          $deleteFollow->delete();
        }
      }
    }

    // Create comment.
    if ($activity->getType() == 'Create' && $this->configFactory->get('activitypub.settings')->get('comment_create_enable') && !$update) {
      $payload = @json_decode($activity->getPayLoad(), TRUE);
      if (isset($payload['object']) && isset($payload['object']['inReplyTo']) && !empty($payload['object']['inReplyTo'])) {
        $activity->set('config_id', 'inbox_reply');
        $this->activityPubProcessClient->createQueueItem($activity);
      }
    }

  }

  /**
   * {@inheritdoc}
   */
  public function build(ActivityPubActivityInterface $activity, EntityInterface $entity = NULL) {
    $return = [];

    if ($this->getConfiguration()['activity'] == 'Accept') {
      $attributes = [
        'id' => $this->renderEntityUrl($activity),
        'type' => $this->getConfiguration()['activity'],
        'actor' => $activity->getActor(),
        'to' => [$activity->getObject()],
        'object' => [
          'type' => 'Follow',
          'id' => $activity->getExternalId(),
          'actor' => $activity->getObject(),
          'object' => $activity->getActor(),
        ]
      ];
      $return = $attributes;
    }

    if ($this->getConfiguration()['activity'] == 'Delete') {

      $to = [ActivityPubActivityInterface::PUBLIC_URL];
      if (!empty($activity->getObject())) {
        $to[] = $activity->getObject();
      }

      /** @var \Drupal\activitypub\Entity\ActivityPubActorInterface $actor */
      $actor = $this->entityTypeManager->getStorage('activitypub_actor')->loadActorByEntityIdAndType($activity->getOwnerId(), 'person');
      $cc = [$this->renderUrl(Url::fromRoute('activitypub.followers', ['user' => $actor->getOwnerId(), 'activitypub_actor' => $actor->getName()], ['absolute' => TRUE]))];

      $attributes['to'] = $to;
      $attributes['cc'] = $cc;

      if (isset($entity)) {
        $id = $this->renderEntityUrl($entity);
      }
      else {
        $id = $this->renderEntityUrl($activity);
      }

      $attributes = [
        'type' => $this->getConfiguration()['activity'],
        'actor' => $activity->getActor(),
        'to' => $to,
        'cc' => $cc,
        'id' => $id,
        'object' => ['id' => $id],
      ];
      $type = Type::create($attributes);
      $return = $type->toArray();
    }

    return $return;
  }

  /**
   * {@inheritdoc}
   */
  public function doInboxProcess(ActivityPubActivityInterface $activity, EntityInterface $entity = NULL) {
    if ($activity->getType() == 'Create' && $this->configFactory->get('activitypub.settings')->get('comment_create_enable')) {
      $payload = @json_decode($activity->getPayLoad(), TRUE);
      if (isset($payload['object']) && isset($payload['object']['inReplyTo']) && !empty($payload['object']['inReplyTo'])) {
        /** @noinspection PhpUnhandledExceptionInspection */
        // The exception is caught in the process client.
        $this->handleCommentCreation($activity);
      }
    }
  }

  /**
   * Handle comment creation.
   *
   * @param \Drupal\activitypub\Entity\ActivityPubActivityInterface $activity
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function handleCommentCreation(ActivityPubActivityInterface $activity) {
    $pid = 0;

    $replyPath = trim(parse_url($activity->getObject(), PHP_URL_PATH));
    if (empty($replyPath)) {
      $replyPath = "/";
    }

    $path = \Drupal::service('path_alias.manager')->getPathByAlias($replyPath);
    $params = Url::fromUri("internal:" . $path)->getRouteParameters();

    // This can be a reply on a comment. Get the node to attach the
    // comment there and set pid.
    if (!empty($params) && key($params) == 'comment') {
      /** @var \Drupal\comment\CommentInterface $comment */
      $comment = $this->entityTypeManager->getStorage('comment')->load($params['comment']);
      if ($comment && $comment->getCommentedEntityTypeId() == 'node') {
        $pid = $comment->id();
        $params = ['node' => $comment->getCommentedEntityId()];
      }
    }

    // We currently support only comments on nodes.
    if (!empty($params) && key($params) == 'node') {
      $entity_type = key($params);

      $storage = $this->entityTypeManager->getStorage($entity_type);
      if ($storage) {

        $payload = @json_decode($activity->getPayLoad(), TRUE);
        $object = $payload['object'];

        $comment_type = 'comment';
        $activity_reference_field = 'activitypub_activity';
        $node_comment_field_name = 'comment';

        /** @var \Drupal\node\NodeInterface $node */
        $node = $storage->load($params[$entity_type]);
        if ($node && $node->hasField($node_comment_field_name) && $node->{$node_comment_field_name}->status == 2) {

          $subject = 'Reply by ' . $payload['actor'];

          $values = [
            'subject' => $subject,
            'name' => $payload['actor'],
            'status' => 1,
            'entity_id' => $node->id(),
            'entity_type' => 'node',
            'pid' => $pid,
            'comment_body' => [
              'value' => $object['content'],
              'format' => Settings::get('activitypub_comment_format', 'restricted_html'),
            ],
            'comment_type' => $comment_type,
            'field_name' => $node_comment_field_name,
            $activity_reference_field => [
              'target_id' => $activity->id(),
            ],
          ];

          $comment = $this->entityTypeManager->getStorage('comment')->create($values);
          $comment->setCreatedTime($activity->getCreatedTime());
          $comment->save();
        }
      }
    }
  }


}
