<?php

namespace Drupal\activitypub\Services;

use ActivityPhp\Server;
use Drupal\activitypub\Entity\ActivityPubActorInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\user\UserInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class ActivityPubUtility implements ActivityPubUtilityInterface {

  /**
   * The image style storage.
   *
   * @var \Drupal\image\ImageStyleStorageInterface
   */
  protected $imageStyleStorage;

  /**
   * The actor storage.
   *
   * @var \Drupal\activitypub\Entity\Storage\ActivityPubActorStorageInterface
   */
  protected $actorStorage;

  /**
   * The activitypub storage.
   *
   * @var \Drupal\activitypub\Entity\Storage\ActivityPubActivityStorageInterface
   */
  protected $activityStorage;

  /**
   * The request stack
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack $requestStack
   */
  protected $requestStack;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * ActivityPubUtility constructor
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RequestStack $request_stack, ConfigFactoryInterface $config_factory) {
    $this->imageStyleStorage = $entity_type_manager->getStorage('image_style');
    $this->actorStorage = $entity_type_manager->getStorage('activitypub_actor');
    $this->activityStorage = $entity_type_manager->getStorage('activitypub_activity');
    $this->requestStack = $request_stack;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function getActivityPubName(ActivityPubActorInterface $actor) {
    return $actor->getName() . '@' . $this->requestStack->getCurrentRequest()->getHost();
  }

  /**
   * {@inheritdoc}
   */
  public function getActivityPubID(ActivityPubActorInterface $actor) {
    return $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost() . '/user/' . $actor->getOwnerId() . '/activitypub/' . $actor->getName();
  }

  /**
   * {@inheritdoc}
   */
  public function getActivityPubUserAvatar(UserInterface $user) {
    $config = $this->configFactory->get('activitypub.settings');
    $avatar_user_field = $config->get('avatar_user_field');
    /** @var \Drupal\file\FileInterface $image */
    if (!empty($avatar_user_field) && $user->hasField($avatar_user_field) && ($image = $user->get($avatar_user_field)->entity)) {

      /** @var \Drupal\image\ImageStyleInterface $image_style */
      $image_style = $this->imageStyleStorage->load($config->get('avatar_user_style'));
      $image_url = $image_style->buildUrl($image->getFileUri());

    }
    else {
      $avatar_path = activitypub_default_avatar_path();
      if ($avatar_path == 'assets/avatar.png') {
        $avatar_path = drupal_get_path('module', 'activitypub') . '/' . $avatar_path;
      }
      $image_url = file_create_url($avatar_path);
    }

    return $image_url;
  }

  /**
   * {@inheritdoc}
   */
  public function getServer(array $config = []) {
    $config += [
      'cache' => [
        'ttl' => activitypub_cache_ttl(),
        'stream' => activitypub_cache_path()
      ],
    ];
    $config['instance']['types'] = 'ignore';
    return new Server($config);
  }

  /**
   * {@inheritdoc}
   */
  public function alterNodeInfo(array &$data) {
    $data['protocols'] = ['activitypub'];
    $data['usage']['users']['total'] = $this->actorStorage->getActorCount(['status' => 1]);
    $data['usage']['localPosts'] = $this->activityStorage->getActivityCount(['collection' => ['outbox', 'liked'], 'status' => 1]);
  }

  /**
   * {@inheritdoc}
   */
  public function onUserDelete(UserInterface $account) {
    $actors = $this->actorStorage->loadByProperties(['uid' => $account->id()]);
    $this->actorStorage->delete($actors);
  }

  /**
   * {@inheritdoc}
   */
  public function onActorDelete(ActivityPubActorInterface $actor) {
    $activities = $this->activityStorage->loadByProperties(['uid' => $actor->getOwnerId()]);
    $this->activityStorage->delete($activities);
  }

}
