<?php

namespace Drupal\Tests\activitypub\Functional;

use Drupal\Core\Url;

/**
 * Tests Activity functionality.
 *
 * @group activitypub
 */
class ActivityTest extends ActivityPubTestBase {

  /**
   * Test activity request.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testActivityStorage() {
    $assert_session = $this->assertSession();
    $page = $this->getSession()->getPage();

    $this->enableActivityPub($assert_session, TRUE);
    $this->drupalLogout();
    $actor_href = Url::fromRoute('activitypub.user.self', ['user' => $this->authenticatedUserOne->id(), 'activitypub_actor' => $this->accountNameOne], ['absolute' => TRUE])->toString();
    $object_href = Url::fromRoute('activitypub.user.self', ['user' => $this->authenticatedUserTwo->id(), 'activitypub_actor' => $this->accountNameTwo], ['absolute' => TRUE])->toString();

    $payload = [
      'type' => 'Follow',
      '@context' => 'https://www.w3.org/ns/activitystreams',
      'id' => $actor_href . '/follow/1',
      'actor' => $actor_href,
      'object' => $object_href,
    ];
    $inbox = Url::fromRoute('activitypub.inbox', ['user' => $this->authenticatedUserTwo->id(), 'activitypub_actor' => $this->accountNameTwo])->toString();
    $response = $this->sendInboxRequest($inbox, $payload);
    self::assertEquals(202, $response->getStatusCode());

    /** @var \Drupal\activitypub\Entity\Storage\ActivityPubActorStorageInterface $storage */
    $storage = \Drupal::entityTypeManager()->getStorage('activitypub_activity');
    /** @var \Drupal\activitypub\Entity\ActivityPubActivityInterface $activity */
    $activity = $storage->load(1);
    self::assertEquals("Follow", $activity->getType());
    self::assertEquals($payload['id'], $activity->getExternalId());
    self::assertEquals($actor_href, $activity->getActor());
    self::assertEquals($object_href, $activity->getObject());

    $count = \Drupal::queue(ACTIVITYPUB_OUTBOX_QUEUE)->numberOfItems();
    self::assertEquals(1, $count);

    // Get followers of account 1.
    $this->drupalGet('user/' . $this->authenticatedUserOne->id() . '/activitypub/' . $this->accountNameOne . '/followers');
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(0, $content->totalItems);

    // Get following of account 1 - we don't support that yet, but let's make
    // the call already to see that it doesn't fatal.
    $this->drupalGet('user/' . $this->authenticatedUserOne->id() . '/activitypub/' . $this->accountNameOne . '/following');
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(0, $content->totalItems);

    // Get following of account 1 - we don't support that yet, but let's make
    // the call already to see that it doesn't fatal.
    $this->drupalGet('user/' . $this->authenticatedUserOne->id() . '/activitypub/' . $this->accountNameOne . '/following', ['query' => ['page' => 0]]);
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(0, count($content->orderedItems));

    // Get followers of account 2 - we don't support that yet, but let's make
    // the call already to see that it doesn't fatal.
    $this->drupalGet('user/' . $this->authenticatedUserTwo->id() . '/activitypub/' . $this->accountNameTwo . '/followers');
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(1, $content->totalItems);

    // Get following of account 1 - we don't support that yet, but let's make
    // the call already to see that it doesn't fatal.
    $this->drupalGet('user/' . $this->authenticatedUserTwo->id() . '/activitypub/' . $this->accountNameTwo . '/following');
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(0, $content->totalItems);

    // Get followers of account 2.
    $this->drupalGet('user/' . $this->authenticatedUserTwo->id() . '/activitypub/' . $this->accountNameTwo . '/followers', ['query' => ['page' => 0]]);
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals($content->orderedItems[0], $actor_href);

    // Allow to block incoming domains.
    $this->drupalLogin($this->authenticatedUserTwo);
    $edit = [
      'activitypub_blocked_domains' => 'http://blocked.domain*',
    ];
    $this->drupalPostForm('user/' . $this->authenticatedUserTwo->id() . '/activitypub/settings', $edit, 'Save');
    $assert_session->responseContains('http://blocked.domain*');
    $this->drupalLogout();

    $payload = [
      'type' => 'Follow',
      '@context' => 'https://www.w3.org/ns/activitystreams',
      'id' => $actor_href . '/follow/2',
      'actor' => 'http://blocked.domain/user/spammer',
      'object' => $object_href,
    ];
    $response = $this->sendInboxRequest($inbox, $payload);
    self::assertEquals(403, $response->getStatusCode());

    $payload['actor'] = 'http://allowed.domain/user/good';
    $payload['id'] = 'http://allowed.domain/user/good/follow/2';
    $payload['object'] = (object) [
      'object' => $object_href,
    ];
    $response = $this->sendInboxRequest($inbox, $payload);
    self::assertEquals(202, $response->getStatusCode());
    $activity = $storage->load(3);
    self::assertEquals("Follow", $activity->getType());
    self::assertEquals($payload['actor'], $activity->getActor());
    self::assertEquals($object_href, $activity->getObject());

    $count = \Drupal::queue(ACTIVITYPUB_OUTBOX_QUEUE)->numberOfItems();
    self::assertEquals(2, $count);

    $activity = $storage->load(4);
    self::assertEquals("outbox", $activity->getCollection());
    self::assertEquals("Accept", $activity->getType());
    self::assertEquals($payload['actor'], $activity->getObject());
    self::assertEquals($object_href, $activity->getActor());

    // Get followers of account 2.
    $this->drupalGet('user/' . $this->authenticatedUserTwo->id() . '/activitypub/' . $this->accountNameTwo . '/followers', ['query' => ['page' => 0]]);
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(2, count($content->orderedItems));

    // Remove follower (via undo).
    $payload['type'] = 'Undo';
    $payload['id'] = 'http://allowed.domain/user/good/follow/3';
    $payload['actor'] = 'http://allowed.domain/user/good';
    $payload['object'] = (object) [
      'type' => 'Follow',
      'actor' => $payload['actor'],
      'object' => $object_href,
    ];
    $response = $this->sendInboxRequest($inbox, $payload);
    self::assertEquals(202, $response->getStatusCode());

    // Get followers of account 2.
    $this->drupalGet('user/' . $this->authenticatedUserTwo->id() . '/activitypub/' . $this->accountNameTwo . '/followers', ['query' => ['page' => 0]]);
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(1, count($content->orderedItems));

    $this->drupalGet('nodeinfo/content');
    $content = json_decode($page->getContent());
    self::assertEquals(2, $content->usage->users->total);
    self::assertEquals(0, $content->usage->localPosts);

    // Send a DELETE where actor and object are equal, this activity will be
    // deleted in the end.
    $payload = [
      'type' => 'Delete',
      'id' => 'https://mastodon.social/user/deleted#delete',
      'actor' => 'https://mastodon.social/user/deleted',
      'object' => 'https://mastodon.social/user/deleted',
    ];
    $response = $this->sendInboxRequest($inbox, $payload);
    self::assertEquals(202, $response->getStatusCode());
    $ids = $storage->getQuery()->condition('uid', $this->authenticatedUserTwo->id())->execute();
    self::assertEquals(4, count($ids));

    // Make sure there's only one follower.
    $this->clearQueue(ACTIVITYPUB_OUTBOX_QUEUE);
    $payload = [
      'type' => 'Follow',
      '@context' => 'https://www.w3.org/ns/activitystreams',
      'id' => $actor_href . '/follow/1',
      'actor' => $actor_href,
      'object' => $object_href,
    ];
    $inbox = Url::fromRoute('activitypub.inbox', ['user' => $this->authenticatedUserTwo->id(), 'activitypub_actor' => $this->accountNameTwo])->toString();
    $response = $this->sendInboxRequest($inbox, $payload);
    self::assertEquals(202, $response->getStatusCode());
    $this->drupalGet('user/' . $this->authenticatedUserTwo->id() . '/activitypub/' . $this->accountNameTwo . '/followers', ['query' => ['page' => 0]]);
    $assert_session->statusCodeEquals(200);
    $content = json_decode($page->getContent());
    self::assertEquals(1, count($content->orderedItems));
    $ids = $storage->getQuery()->condition('uid', $this->authenticatedUserTwo->id())->execute();
    self::assertEquals(4, count($ids));
    $count = \Drupal::queue(ACTIVITYPUB_OUTBOX_QUEUE)->numberOfItems();
    self::assertEquals(0, $count);

  }

}
